package com.nisum.webfluxpractice.controllerWebFlux;

import com.nisum.webfluxpractice.model.Student;
import com.nisum.webfluxpractice.service.StudentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;


@CrossOrigin(origins = "*")
@RequestMapping("/api")
@RestController
public class StudentControllerWebFlux {

    @Autowired
    private StudentService studentService;

    @GetMapping(value = "/students", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<Student> findAll() {
        Flux<Student> studentFlux = studentService.findAll();
        return studentFlux;
    }

    @PostMapping("/students")
    @ResponseStatus(HttpStatus.CREATED)
    public void create(@RequestBody Student s) {
        studentService.create(s);
    }

    @GetMapping("/students/{id}")
    public ResponseEntity<Mono<Student>> findById(@PathVariable("id") Integer id) {
        Mono<Student> s = studentService.findById(id);
        HttpStatus status = s != null ? HttpStatus.OK : HttpStatus.NOT_FOUND;
        return new ResponseEntity<Mono<Student>>(s, status);
    }

    @PutMapping("/students")
    @ResponseStatus(HttpStatus.OK)
    public Mono<Student> update(@RequestBody Student s) {
        return studentService.update(s);
    }

    @DeleteMapping("/students/{id}")
    @ResponseStatus(HttpStatus.OK)
    public void delete(@PathVariable("id") Integer id) {
        studentService.delete(id).subscribe();
    }


}
